/*  File: analyser-gap.h
 *  Project: Deductor
 *  Created by D. Soper on 21 November 2014
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __analyser_AtlasGap_h__
#define __analyser_AtlasGap_h__

/*   std includes   */

/*   project headers  */
#include <user.h>

/*   other includes   */
#include "fastjet.h"


/*   It is not a library just a module. We can use using namespace...  */
using namespace std;
using namespace duct;
using namespace duct::distpoint;
using namespace fjcore;



/*   
 *   This is the declaration of the analyser class.
 */

/*   Narrowing the basic_user template   */
using UserVoid = basic_user<hhc, double, void>;
using UserPlain = basic_user<hhc, double, hist1d>;
using UserRatio = basic_user<hhc, std::array<double, 2>, hist1d,
                   sample_ratio_traits<std::array<double, 2>>>;



class UserAtlasGap : public basic_user_set<UserRatio>
{
public:
  /*   constructors   */
  UserAtlasGap(JetAlgorithm jet_algorithm=antikt_algorithm, double Rparam=0.6,
           RecombinationScheme recomb_scheme=E_scheme, Strategy strategy=Best)
  : jetDef(jet_algorithm, Rparam, recomb_scheme, strategy) {}

  /*   User defined init function. It is called once
   *   at the begining of the calculation.
   */
  void initfunc();

  /*   The analyser routine. It analyses the events and fill the histograms.   */
  void userfunc(const shower_history<hhc>&);
  
private:
  /*   For fastjet analysis - select algorithm and parameters.
   *   The UserJets constructor above initializes jetDef.
   */
   JetDefinition jetDef;
  
  /*  Predefined vectors of Pseudojets  */
  mutable std::vector<PseudoJet> theparticles;
  mutable std::vector<PseudoJet> incJets0;
  mutable std::vector<PseudoJet> incJets;
};



#endif


