/*  File: fastjet.h
 *  Project: Deductor
 *  Created by Zoltán Nagy on 10/30/13.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __analyser_fastjet_h__
#define __analyser_fastjet_h__ 1

/*   std includes  */

/*   project headers   */
#include <event.h>

/*   other includes   */
#include "fjcore.hh"


inline fjcore::PseudoJet make_pseudojet(const duct::lorentzvector<double>& p) {
  return fjcore::PseudoJet(p.X(), p.Y(), p.Z(), p.T());
}


template<class _Tag>
void make_pseudojets(std::vector<fjcore::PseudoJet>& pj, const duct::event<_Tag>& p)
{
  /*   number of partons   */
  int np =  static_cast<int>(p.template number_of<duct::qcd_out>());
  
  /*  resiing the array  */
  pj.resize(np);
  
  /*   copying the momenta   */
  for(int i = 0; i < np; i++)
    pj[i] = make_pseudojet(p[i+1].momentum);
}




#endif
