/*  File: analyser-Color.cc
 *  Project: Deductor
 *  Created by D. Soper on 6 March 2014
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   std includes   */

/*   project headers  */

/*   local headers  */
#include "analyser-Color.h"

/*   other includes   */

void UserColor::initfunc()
{
  /*   Create a histogram to calculate the color distribution   */
  unsigned int nbins = 52;
  double Nmin = -0.5,
         Nmax =  51.5;
  
  UserVoid::phys(1, "Jet cross section");
  
  UserPlain::phys(0, "Color suppression 0",
                  spacing<hist1d>::linear(nbins, Nmin, Nmax));
  UserPlain::phys(2, "Color suppression 1 or 2",
                  spacing<hist1d>::linear(nbins, Nmin, Nmax));
  UserPlain::phys(4, "Color suppression 3 or 4",
                  spacing<hist1d>::linear(nbins, Nmin, Nmax));
  UserPlain::phys(6, "Color suppression 5 or 6",
                  spacing<hist1d>::linear(nbins, Nmin, Nmax));

  UserPlain::phys(10,"Distribution of |weights|",
                  spacing<hist1d>::logarithmic(30, 1.0e-20, 1.0e10));
  
  UserPlain::phys(11,"Distribution of PTstart",
                  spacing<hist1d>::linear(50, 100.0, 600.0));
  
  UserPlain::phys(12,"Distribution of number of particles m",
                  spacing<hist1d>::linear(72, -0.5, 71.5));
  
  cout<< "UserColor is using jet algorithm "
  <<jetDef.jet_algorithm()
  <<" with R = "<<jetDef.R()<<endl;
}



void UserColor::userfunc(const shower_history<hhc>& s)
{
  /*   the shower stage after the shower evolution   */
  auto& sb = s.back();
  auto& p = sb.state;
  /*   the shower stage before the shower evolution   */
  auto& sf = s.front();
  auto& pf = sf.state;

  // Other shower stages
  auto is = s.begin();
  auto send = s.end();
  int I[52];
  for(int k = 0; k <= 50; k++)
    I[k] = (is != send ?
            (is++)->state.color_suppression_index
            : p.color_suppression_index);
  I[51] = p.color_suppression_index;
  /*   Calculate the weight of the event   */
  double w = sb.weight*color_weight(sb);
  /*   Converting deductor event to FastJet format   */
  make_pseudojets(theparticles, p);
  
  /*    Run fastjet algorithm   */
  ClusterSequence clustSeq1(theparticles, jetDef);
  
  /*    Extract inclusive jets (above pTjetmin)   */
  double pTjetmin = 200.0_GeV;
  incJets = clustSeq1.inclusive_jets(pTjetmin);
  
  /*    Fill histogram.   */
  
  /*    Only count jets that have |y| < 2.0   */
  /*    and 500 < PT                          */
  for(unsigned int j = 0; j < incJets.size(); j++)
    if(abs(incJets[j].rap()) <= 2.0
       && incJets[j].perp()  >= 500)
  {
    UserVoid::physfill(1, w);
    double wbinsize = 0.0;
    for (int j = -30; j <= 12; j++)
    {
      if (pow(10.0,j) < abs(w) &&  abs(w) <= pow(10.0,j+1))
      {
        wbinsize = 0.9*pow(10.0,j+1);
      }
      
    }
    int nn = 51;
    UserPlain::physfill(10, wbinsize, dirac(), abs(w));
    double PTstart = pf[1].momentum.perp();
    double PTjet = incJets[j].perp();
    UserPlain::physfill(11,w,dirac(),PTstart);
    if (abs(w) > 0.1) {
      cout << "Weight = " << w << " for I["<<nn<<"] = " << I[nn]
           << " and for PTjet = " << PTjet
           << " and PTstart = " << PTstart << endl;
    }
    
    UserPlain::physfill(12, w, dirac(), p.number_of<qcd_out>());
    
    for (unsigned int n = 0; n <= 51; n++)
    {
      if (I[n] == 1) I[n] = 2;
      if (I[n] == 3) I[n] = 4;
      if (I[n] == 5) I[n] = 6;
        
      UserPlain::physfill(I[n], w, dirac(), n);
    }
  }
  
  
}

