/*  File: analyser-jets.cc
 *  Project: Deductor
 *  Created by Zoltán Nagy on 9/8/13.
 *  Modified by D. Soper on 17 December 2013
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   std includes   */

/*   project headers  */

/*   local headers  */
#include "analyser-jets.h"

/*   other includes   */

void UserJets::initfunc()
{
  /*   Create an object to calculate the total cross section   */
  UserVoid::phys(1,"Total cross section (hard)");
  UserVoid::phys(2,"Total cross section");
  
  /*   Create a histogram to calculate the one jet cross section   */
  unsigned int nbins = 60;
  double pTmin = 200.0_GeV, pTmax = 800.0_GeV;
  auto bins = spacing<hist1d>::linear(nbins, pTmin, pTmax);
  
  UserPlain::phys(1, "One jet inclusive cross section", bins);
  UserPlain::phys(2, "One jet inclusive cross section at hard interaction!",bins);
  
  cout << "UserJets is using jet algorithm " << jetDef.jet_algorithm() << " with R = " << jetDef.R() << endl;
}



void UserJets::userfunc(const shower_history<hhc>& s)
{
  /*   the shower stage before the shower evolution
   *   aka hard state
   */
  auto& sf = s.front();
  auto& pf = sf.state;

  /*   the shower stage after the shower evolution   */
  auto& sb = s.back();
  auto& p = sb.state;
  
  /*   Calculate the weight of the event   */
  constexpr double tonb = 389379.338;
  
  double wf = sf.weight*color_weight(sf)*tonb;
  double w = sb.weight*color_weight(sb)*tonb;
  
  /*   Total cross sections   */
  UserVoid::physfill(1, wf);
  UserVoid::physfill(2, w);
  
  
  //   One jet inclusive cross section after showering
  // --------------------------------------------------
  
  /*   Converting deductor event to a FastJet format   */
  make_pseudojets(theparticles, p);
  
  /*    Run fastjet algorithm   */
  ClusterSequence clustSeq1(theparticles, jetDef);
  
  /*    Extract inclusive jets sorted by pT (above pTjetmin)   */
  double pTjetmin = 200.0_GeV;
  incJets = clustSeq1.inclusive_jets(pTjetmin);
  
  /*      Fill histogram,  only count jets that have |y| < 2.0  */
  for(unsigned j = 0; j < incJets.size(); j++)
    if(abs(incJets[j].rap()) < 2.0)
      UserPlain::physfill(1, w, dirac(),incJets[j].perp());


  //   One jet inclusive cross section before showering
  // --------------------------------------------------
  
  /*   Fill histogram,  only count jets that have |y| < 2.0
   *   The two jets are back-to-back, since we have only two 
   *   partons in the final state.
   */
  if(abs(pf[1].momentum.rapidity()) < 2.0)
    UserPlain::physfill(2, wf, dirac(), pf[1].momentum.perp());
  
  if(abs(pf[2].momentum.rapidity()) < 2.0)
    UserPlain::physfill(2, wf, dirac(), pf[2].momentum.perp());
}

